@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
-# -*- mode: python -*-
 # -*- coding: utf-8 -*-
+# -*- mode: python -*-
 
 """General purpose workflow of tasks with dependencies.
 
@@ -83,6 +83,11 @@ TaskState = base.MakeTuple('TaskState',
 )
 
 
+DOWNSTREAM = 'downstream'
+UPSTREAM = 'upstream'
+
+
+
 def GetTaskID(task_or_id):
   """Gets the ID of a task, given a parameter that is either a Task or an ID.
 
@@ -332,6 +337,10 @@ class Task(object, metaclass=abc.ABCMeta):
     """Returns: the end time of the task run, or None if not completed yet."""
     return self._end_time
 
+  def GetGraphvizLabel(self):
+    """Returns: a label for the Graphiz node representing this task."""
+    return self.task_id
+
 
 # ------------------------------------------------------------------------------
 
@@ -755,12 +764,25 @@ class Workflow(object):
       deps='\n'.join(deps),
     )
 
-  def DumpRunStateAsDot(self):
+  @staticmethod
+  def _GetTaskLabel(task):
+    return task.GetGraphvizLabel()
+
+  def DumpRunStateAsDot(
+      self,
+      make_task_label=None,
+  ):
     """Dumps this workflow as a Graphviz/Dot definition.
 
+    Args:
+      make_task_label: Optional function: task -> task node label.
+          Default is to use Task.MakeDotLabel().
     Returns:
       A Graphviz/Dot definition for this workflow.
     """
+    if make_task_label is None:
+      make_task_label = self._GetTaskLabel
+
     def MakeNode(task):
       task_id = task.task_id
       if task.state == TaskState.FAILURE:
@@ -772,7 +794,7 @@ class Workflow(object):
       else:
         color = 'black'
 
-      label = '\\n'.join(task_id.split('.'))
+      label = make_task_label(task)
       return ('  %s [color="%s", label="%s"];'
               % (base.MakeIdent(task_id), color, label))
 
@@ -887,20 +909,27 @@ class Workflow(object):
           raise
         return svg_file.read().decode()
 
-  def Prune(self, tasks):
+  def Prune(self, tasks, direction):
     """Prunes the workflow according to a sub-set of required tasks.
 
     Args:
       tasks: Collection of tasks to keep.
           Tasks that are not in this set or not required transitively
-          through upstream dependencies of this set are discarded.
+          through upstream/downstream dependencies of this set are discarded.
+      direction: Either DOWNSTREAM or UPSTREAM.
     """
     assert not self._started
 
     # Exhaustive list of tasks to keep:
-    tasks = GetUpstreamTasks(flow=self, tasks=tasks)
+    if direction == UPSTREAM:
+      tasks = GetUpstreamTasks(flow=self, tasks=tasks)
+    elif direction == DOWNSTREAM:
+      tasks = GetDownstreamTasks(flow=self, tasks=tasks)
+    else:
+      raise Error('Invalid filtering direction: %r' % direction)
     keep_ids = frozenset(map(lambda task: task.task_id, tasks))
 
+    # IDs of the tasks to remove:
     remove_ids = set(self._tasks.keys())
     remove_ids.difference_update(keep_ids)
 
@@ -913,6 +942,11 @@ class Workflow(object):
         self._deps))
     self._deps.difference_update(remove_deps)
 
+    # Update task descriptors:
+    for task in self._tasks.values():
+      task._runs_after.difference_update(remove_ids)
+      task._runs_before.difference_update(remove_ids)
+
 
 # ------------------------------------------------------------------------------
 
@@ -1163,14 +1197,14 @@ class WorkflowHTTPMonitor(base.MultiThreadedHTTPServer):
   def __init__(
       self,
       interface='0.0.0.0',
-      port=8000,
+      port=0,
       workflow=None,
   ):
     """Creates a new HTTP endpoint to monitor a workflow.
 
     Args:
-      interface: TCP interface to listen on.
-      port: TCP port to listen on.
+      interface: TCP interface to listen on. 0 or empty means all interfaces.
+      port: TCP port to listen on. 0 means pick a random free port.
       workflow: Optional workflow to monitor.
           Can be set or updated later with SetWorkflow().
     """
@@ -1191,9 +1225,8 @@ class WorkflowHTTPMonitor(base.MultiThreadedHTTPServer):
 
   def Start(self):
     self._thread.start()
-    logging.info(
-        'Workflow monitor started on http://%s:%s',
-        self._interface, self.server_port)
+    logging.info('Workflow monitor started on http://%s:%s',
+                 self.server_name, self.server_port)
 
   def Stop(self):
     self.shutdown()
